<?php
/**
 * Class WC_ClipTransparent
 *
 * @package  Conexa\ClipTransparent\Gateway\WC_ClipTransparent
 */

namespace Conexa\ClipTransparent\Gateway;

use Conexa\ClipTransparent\Includes\Helper\Helper;
use Conexa\ClipTransparent\Sdk\ClipTransparentSdk;

defined( 'ABSPATH' ) || class_exists( '\WC_Payment_Gateway' ) || exit();

/**
 * Main Class Clip Transparent Payment.
 */
class WC_ClipTransparent extends \WC_Payment_Gateway {

	/**
	 * Constructor for the gateway.
	 */
	public function __construct() {
		$this->id                 = \ClipTransparent::GATEWAY_ID;
		$this->has_fields = true; // in case you need a custom credit card form
		$this->method_title       = __( 'Clip Mexico Payments for WooCommerce (Transparent Checkout)', 'clip-transparent' );
		$this->method_description = __( 'Accept all local Mexican and international card payments with Clip (Credit, Debit, Vale cards, Months without interest, Visa, MC, AMEX, Discover, Diners Club, Si Vale, Edenred, Carnet and more).', 'clip-transparent' );

		$this->icon         = Helper::get_assets_folder_url() . '/img/logotype_clip_primary.svg';

		$this->sdk = new clipTransparentSdk(
			Helper::get_option( 'api_key' ),
			Helper::get_option( 'api_secret' )
		);

		// gateways can support subscriptions, refunds, saved payment methods,
		$this->supports = array(
			'products'
		);

		// Define user set variables.
		$this->title        = __( 'Pay with debit and credit card', 'clip-transparent' );
		$this->instructions = $this->get_option(
			$this->description,
			$this->method_description
		);

        // Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		$this->supports[] = 'refunds';


		global $current_section;
		if ( \ClipTransparent::GATEWAY_ID === $current_section ) {

			$this->enqueue_settings_js();

		}

		add_action(
			'woocommerce_update_options_payment_gateways_' . $this->id,
			array(
				$this,
				'process_admin_options',
			)
		);
		add_action(
			'woocommerce_thankyou_' . $this->id,
			array(
				$this,
				'thankyou_page',
			)
		);
	}


	/**
	 * Enqueue_settings_js
	 */
	private function enqueue_settings_js() {
		?>
		<style>
			.logotype_clip {
				width: 30px;
				height: auto;
				position: relative;
				bottom: -8px;
				border-right: 10px solid transparent;
			}
		</style>
		<?php
		wc_enqueue_js(
			'var url = "' . esc_url( Helper::get_assets_folder_url() . '/img/logotype_clip_primary.svg' ) . '";
			jQuery(document).ready(
				function( $ ) {
					$(".logotype_clip").remove();
					var image = new Image();
					image.src = url;
					image.classList.add("logotype_clip");
					$("h2", $(".wrap.woocommerce")).first().prepend(image);
				});'
		);
	}


	/**
	 * Initialize Gateway Settings Form Fields
	 */
	public function init_form_fields()
	{
		$this->form_fields = include 'settings.php';
	}

	/**
	 * You will need it if you want your custom credit card form
	 */
	public function payment_fields() {

		$data = array();
		$data['id'] = $this->id;
		$data['description'] = $this->description;

		$data['sdk'] = \ClipTransparent::get_url( 'sdk' );
        $data['api_key'] = Helper::get_option( 'api_key' );
		$data['theme_options'] = Helper::get_option( 'theme_options' );
        $data['locale_options'] = Helper::get_option( 'locale_options' );
		
		helper::get_template_part('page', 'payment',  $data);
	 
	}

	/*
	* We're processing the payments here
	*/
	public function process_payment( $order_id ) {

		try {

			$nonce_value    = wc_get_var($_REQUEST['woocommerce-process-checkout-nonce'], ''); // phpcs:ignore
			if (empty($nonce_value) || !wp_verify_nonce($nonce_value, 'woocommerce-process_checkout')) {
				WC()->session->set('refresh_totals', true);
				throw new \Exception(__('We were unable to process your order, please try again.', 'clip-transparent'));
			}

			if(isset($_POST['clip_transparent_error']) && !empty($_POST['clip_transparent_error'])) {
				$message = "";

				if ( 'CL2200' === $_POST[ 'clip_transparent_error' ] ) {
					$message = __( 'Check the card details', 'clip-transparent' ) ;     
				}
				if ( 'AI1300' === $_POST[ 'clip_transparent_error' ] ) {
					$message = __( 'Try another card', 'clip-transparent' ) ;     
				}
				if ( 'GATEWAY TIMEOUT' === $_POST[ 'clip_transparent_error' ] ) {
					$message = __( 'Internal Server Error', 'clip-transparent' ) ;     
				}

				throw new \Exception(__($message));
			}
	
			if( empty( $_POST[ 'clip_transparent_cardtoken' ] ) ) {
				throw new \Exception(__('We were unable to process your order, please try again.', 'clip-transparent'));
			}



			// we need it to get any order detailes
			$order = wc_get_order( $order_id );
			
			$extradata  = array(
				'clip_transparent_cardtoken' => filter_var($_REQUEST['clip_transparent_cardtoken'], FILTER_SANITIZE_STRING),
				'clip_transparent_installments' => filter_var($_REQUEST['clip_transparent_installments'], FILTER_SANITIZE_STRING),
				'clip_transparent_prevention' => filter_var($_REQUEST['clip_transparent_prevention'], FILTER_SANITIZE_STRING)
			);
			$charge_payment = $this->sdk->charge_payment($order_id, $extradata);

			$response = $this->sdk->get_charge_payment( $charge_payment['id'] );

			$res = $this->handle_response($order_id, $response);

			return $res;

		} catch (\Exception $e) {
			wc_add_notice($e->getMessage(), 'error');
		}
	}



	protected function handle_response(int $order_id, array $data)
	{

		if (isset($data['status']) && 'pending' === $data['status'] && isset($data['pending_action']) && isset($data['pending_action']['url'])) {
			//3ds - Redirect to thank you page			
			return $this->mark_order_as_waiting_3ds($order_id, $data);
		} else if (isset($data['status']) && 'approved' === $data['status']) {
			return $this->mark_order_as_paid($order_id, $data);

		} else if (isset($data['errors']['error_code'])) {
		
			$message = "";

			if ( 'RE-ERI03' === $data['errors']['error_code'] ) {
				$message = __( 'Unfortunately, international cards are not accepted for this payment   . Contact the business 🛍️ to enable international cards, or Use a card issued in Mexico to complete the transaction 🇲🇽', 'clip-transparent' ) ;     
			}
			if ( 'RE-ERI01' === $data['errors']['error_code'] ) {
				$message = __( 'Oops! 😮 Your payment was not made. Dont worry, try again following our suggestions; Use a different card, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' ) ;     
			}
			if ( 'RE-ERI01' === $data['errors']['error_code'] ) {
				$message = __( 'Oops! 😮 Your payment was not made. Dont worry, try again following our suggestions; Use a different card, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' ) ;     
			}
			$re_iss_errors = array('RE-ISS11', 'RE-ISS12', 'RE-ISS13', 'RE-ISS15', 'RE-ISS16', 'RE-ISS17', 'RE-ISS18');
			if (in_array($data['errors']['error_code'], $re_iss_errors)) {
				$message = __('Oops! 😮 Your payment was not made. Dont worry, try again following our suggestions; Use a different card, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' );
			}
			$re_iss_errors = array('RE-ISS06', 'RE-ISS07');
			if (in_array($data['errors']['error_code'], $re_iss_errors)) {
				$message = __('Im sorry! 🙁 your payment could not be completed. Try another card, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' );
			}
			$re_iss_errors = array('RE-ERI02', 'RE-ERI03', 'RE-BIN01');
			if (in_array($data['errors']['error_code'], $re_iss_errors)) {
				$message = __('Im sorry! 🙁 your payment could not be completed. Try another card, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' );
			}
			if ( 'RE-ISS01' === $data['errors']['error_code'] ) {
				$message = __( 'Oops! 😕 it seems that your bank rejected the payment due to lack of funds 💵. Contact your bank to check your card limit and try again, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' ) ;     
			}
			$re_iss_errors = array('RE-ISS14', 'RE-ISS99');
			if (in_array($data['errors']['error_code'], $re_iss_errors)) {
				$message = __('Were sorry, but we encountered a problem processing your payment. Check the status by updating your order or contacting the merchant and try payment again if necessary.', 'clip-transparent' );
			}
			$re_iss_errors = array('RE-ISS02', 'RE-ISS03');
			if (in_array($data['errors']['error_code'], $re_iss_errors)) {
				$message = __('It seems that your bank rejected the payment 🙁. Confirm your card information 💳 and try again, or Choose to pay cash at a nearby convenience store 🏪💸', 'clip-transparent' );
			}

			throw new \Exception(__($message));
			
		}else{
			throw new \Exception(__('We were unable to process your order, please try again.', 'clip-transparent'));
		}
	}

	protected function mark_order_as_waiting_3ds(int $order_id, array $data)
	{

		$order = wc_get_order($order_id);

		$order->update_meta_data(
			\ClipTransparent::META_ORDER_3DS_URL,
			$data['pending_action']['url']
		);
		
		$receipt_no = isset( $data[ 'receipt_no' ] )? $data[ 'receipt_no' ] : '';
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_RECEIPT_NO,
			$receipt_no
		);
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_TRANSACTION_ID,
			$data['id']
		);
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_TRANSACTION_DATE,
			current_datetime()->format('Y-m-d H:i:s')
		);

		$order->save();

		$payment_nonce = wp_create_nonce(\ClipTransparent::GATEWAY_ID);
		// Return to Pay order Page
		return array(
			'result' => 'success',
			'redirect' => add_query_arg('clip_transparent_nonce', $payment_nonce, add_query_arg('clip_transparent_3ds', true, $order->get_checkout_payment_url(true))),
		);
	}

	protected function mark_order_as_paid(int $order_id, array $data)
	{

		$order = wc_get_order($order_id);
		$order->payment_complete();

		$receipt_no = isset( $data[ 'receipt_no' ] )? $data[ 'receipt_no' ] : '';
		if ( !empty( $receipt_no ) ) {
			$order->add_order_note(
				sprintf(
					/* translators: %s: System Flag */
					esc_html__( 'Clip Transparent - Approved Payment. Receipt: %s. ID %s', 'clip-transparent' ),
					$receipt_no,
					$data['id']
				)
			);
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_RECEIPT_NO,
			$receipt_no
		);
		}
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_TRANSACTION_ID,
			$data['id']
		);
		$order->update_meta_data(
			\ClipTransparent::META_ORDER_TRANSACTION_DATE,
			current_datetime()->format('Y-m-d H:i:s')
		);

		$order->save();

		// Return thank you page redirect
		return array(
			'result' => 'success',
			'redirect' => $this->get_return_url($order)
		);
	}

	/**
	 * Output for the order received page.
	 *
	 * @param string $order_id Order Id.
	 */
	public function thankyou_page($order_id)
	{
		// Nothing to add, but required to avoid Warnings.
	}


	/**
	 * Process Refunds
	 * 
	 * @param int $order_id Order to Refund.
	 * @param float $amount Amount to Refund.
	 * @param string $reason
	 * 
	 * @return bool
	 */
	public function process_refund( $order_id, $amount = NULL, $reason = '' ) {      

		if ( empty( $order_id ) ) return false;

		if ( empty( $amount ) ) {
			return new \WP_Error( 'wc-order', sprintf( __( 'Clip Transparent: Skipping ZERO value refund for Order ID %s.', 'clip-transparent'), $order_id ) );						
		}

        $order = wc_get_order($order_id);
        $order_data = $order->get_data();

        $response = $this->sdk->request_refund( $order_id, $amount, $reason );

		if ( isset( $response[ 'code_message' ] ) ) {
			$message = "";
			if ( 'AI1801' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: The refund amount is greater that the original.', 'clip-transparent' ) ;     
			}
			if ( 'AI1802' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: The refund amount, plus previous refunds, is greater than the original amount.' , 'clip-transparent' ) ;     
			}
			if ( 'AI1803' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: The refund date has expired.', 'clip-transparent' ) ;     
			}
			if ( 'AI1804' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: Refund declined.', 'clip-transparent' ) ;     
			}
			if ( 'AI1805' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: Refunds are disabled.', 'clip-transparent' ) ;     
			}
			if ( 'AI1806' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: Refund is disabled for payments with MSI and MCI.', 'clip-transparent' ) ;     
			}
			if ( 'AI1807' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: Refund in process for this transaction. Please try again later.', 'clip-transparent' ) ;     
			}
			if ( 'AI1400' === $response[ 'code_message' ] ) {
				$message = __( 'Clip: Insufficient funds to make the refund.', 'clip-transparent' ) ;     
			}
			return new \WP_Error( 'wc-order', $message );									
		} elseif( ! isset( $response[ 'id' ] ) ) {
			return new \WP_Error( 'wc-order', __( 'Clip: Unauthorized.', 'clip-transparent' ) );		
		} else { 
			$order->add_order_note( sprintf( __( 'Clip: Refund requested. Id: %s .', 'clip-transparent' ), $response[ 'id' ] ) );           
		}
		$order->save(); 		
 
     	return true;

    }


}
